{% extends "main.html" %}


{% block hero %}
  <style>
    .ibis-logo {
      max-height: 1em;
      display: inline;
      margin-left: 1em;
    }

    .ibis.position div {
      font-size: 2em;
    }
  </style>

  {#
  <div class="md-banner" style="background-color: var(--md-primary-fg-color)">
    <div class="md-banner__inner md-grid md-typeset">
        {{ render(page.id, environments=ibis.IBISGraph) }}
    </div>
  </div>
  #}

  <aside class="md-banner ibis position">
    <div class="md-banner__inner md-grid md-typeset">
      {{ render(page.id, environments=[ibis.Breadcrumbs]) }}
    </div>
  </aside>

  {{ super() }}
{% endblock %}
